/* ---------------------------------------------------------------------------
 *  IBM (VGA) fb operators
 * --------------------------------------------------------------------------
 *
 *  Copyright (C) 1997, The Linux-GGI Project
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ---------------------------------------------------------------------------
 *
 *  $Log: vga_fb.inc,v $
 *  Revision 1.5  1997/12/14 21:55:39  becka
 *  uint/sint namespace fixed ... please test ...
 *
 *  Revision 1.4  1997/11/08 15:12:27  becka
 *  Namespace cleanup ... probably broke all and everything ...
 *
 *  Revision 1.3  1997/10/29 20:21:30  becka
 *  New driver - aalib
 *  VGA fixes
 *
 *  Revision 1.2  1997/09/06 04:05:43  taylorj
 *  Major: New maintainer (me).  Also reformatted all VGA driver source to GGI
 *  specs and  converted all debugging prints to use the newer debugging macros.
 *
 *  Revision 1.1  1997/07/01 04:29:47  ezrec
 *  Minor: Alpha attempt at ggiSetSplitline support
 *
 * ---------------------------------------------------------------------------
 *
 *  This code was derived from the following sources of information:
 *  [1]     Xfree 86 VGA driver
 *  [2]     Zen of Graphics Programming (Michael Abrash)
 */

#ifndef _VGA_FB_INC_
#define _VGA_FB_INC_

static void vgafb_set_origin(struct kgi_display *dpy, ggi_uint x, ggi_uint y)
{
	IS_CRITICAL;
	static ggi_uint last_xdot = -1;
	static ggi_uint last_ydot = -1;
	static ggi_uint old_offset = -1;
	register ggi_uint xpix=0,xdot=0;
	register ggi_uint ypix=0,ydot=0;
	register ggi_uint offset;
	ggi_uint fb_width=0;

	switch (dpy->mode.request.graphtype) {
		case GT_TEXT16:
			xpix = x / dpy->mode.request.text.x;
			xdot = x % dpy->mode.request.text.x;
			ypix = y / dpy->mode.request.text.y;
			ydot = y % dpy->mode.request.text.y;
			fb_width = dpy->mode.request.virt.x / 
				dpy->mode.request.text.x * 2;
			break;
		case GT_1BIT:
		case GT_4BIT:
			xpix = x / 8;
			xdot = x % 8;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x / 8;
			break;
		case GT_8BIT:
		/* ModeX and 320x200 planar seem to be the same ... becka */
			xpix = x / 4;
			xdot = (x % 4) * 2;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x / 4;
		break;
		default:
		INTERNALERROR;
	}
		
 	offset = xpix + ypix * fb_width;

	ENTER_CRITICAL;

	if (old_offset != offset) {

		outbCRT(offset >> 8, CRT_STARTADDR_H);
		outbCRT(offset, CRT_STARTADDR_L);
		old_offset = offset;
	}

	if (last_ydot != ydot) {
		outbCRT(ydot & CR08_ROWSCAN_MASK, CRT_PRESETROW);
		last_ydot = ydot;
	}

	if (last_xdot != xdot) {

		register ggi_uint hpan;

		if (dpy->mode.request.graphtype == GT_TEXT16 &&
		    dpy->mode.request.text.x == 9) {
			
			hpan = (xdot == 0) ? 0x08 : (xdot-1) & AR13_DATA_MASK;
		} else {
			
			hpan = xdot & AR13_DATA_MASK;
		}
		WAIT_HVRETRACE();
		outbATC(hpan, ATC_HPAN);
		outb( ATCI_ENABLEDISPLAY, ATCI);
		last_xdot = xdot;
	}
	
	LEAVE_CRITICAL;
}

static void vgafb_set_split(struct kgi_display *dpy, int line)
{
	register uchar foo;

	outbCRT(line & 0xFF, CRT_LINECOMPARE);

	foo = (line & 0x0100)
		? inbCRT(0x07) | CR07_LINECOMPARE_B8
		: inbCRT(0x07) & ~CR07_LINECOMPARE_B8;
	outbCRT(foo, 0x07);

	foo = (line & 0x0200)
		? inbCRT(0x09) | CR09_LINECOMPARE_B9
		: inbCRT(0x09) & ~CR09_LINECOMPARE_B9;
	outbCRT(foo, 0x09);
}

#define	vgafb_set_frame	NULL	/* !!! implement! */

#endif /* _VGA_FB_INC */
