/* ---------------------------------------------------------------------------
**	ICD 2061A / ICS 9161A clock driver
** ---------------------------------------------------------------------------
**
**	Copyright	1995		Andreas Beck
**			1995-1997	Steffen Seeger
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
** ---------------------------------------------------------------------------
**	This code was derived from the following sources of information:
**
**	[1]	IC Designs data sheet for the ICD2061A V2.5, 12-Sep-93
*/

#include <ggi/maintainers.h>
#define	MAINTAINER	Steffen_Seeger
#define	DRIVER_NAME	"ICD 2061A/ICS 9161A clock driver"
#define	DRIVER_REV	"$Revision: 1.9 $"

#define	DEBUG_LEVEL	0

#include <ggi/module.h>
#include "pll.h"

#define	CLOCK_GENERATOR	2

static struct kgi_clock clock =
{
	#ifdef ICS9161A
		"Integrated Circuit Systems",	/* manufacturer		*/
		"ICS 9161A",			/* model		*/
	#else
		"IC Designs",			/* manufacturer		*/ 
		"ICD 2061A ",			/* model		*/
	#endif
	GM_ALL,					/* supported modes 	*/
	0,					/* special capabilities	*/
	{ 0x7FFF, 0x7FFF },			/* maximum resolution	*/

	CLK_TYPE_PROG,				/* clock type		*/
	{{
		400000, 120000000,		/* clock range		*/
	}}
};

static const struct pll_limits dclk_pll = 
{
	14318180,			/* fref				*/
	{ 2, 1 },			/* factor		 	*/
	{ 50000000, 120000000 },	/* fvco range			*/
	{ 0, 7 },			/* post-VCO divider range	*/
	{ 3, 129 },			/* multiplier range		*/
	{ 4, 130 }			/* divider range		*/
};

static const ggi_uint range[] =			/* VCO clock ranges	*/
{
	 50000000,  51000000,  53200000,  58500000,
	 60700000,  64400000,  66800000,  73500000,
	 75600000,  80900000,  83200000,  91500000,
	100000000, 100000000
};

/*	An ICD 2061A command word is as follows:
**
**	 0 -  6		state->div - 2
**	 7 -  9		state->p
**	10 - 16		state->mul - 3 
**	17 - 20		clock index selector
**	21 - 23		register number
*/

#define	INVALID_COMMAND	((ggi_uint) -1)

static ggi_uint pll_command(struct pll_state *s, struct kgi_mode *mode)
{
	ggi_uint clkindex = sizeof(range) / sizeof(range[0]) - 1;

	while ((0 < clkindex) && s->fvco < range[clkindex]) {

		clkindex--;
	}
	ASSERT(range[0] <= s->fvco);

	return (s->div - 2) | (s->p << 7) | ((s->mul - 3) << 10) |
		(clkindex << 17) | (CLOCK_GENERATOR << 21);
}

/*	An icd2061a connected in "standard" fashion is programmed by
**	transferring the command word over the two clock select lines.
**	The chipset driver has to care about how to assert these lines.
*/
#define ICD_BOTH 0x03	/* data and clock	      */
#define ICD_DATA 0x02	/* icd2061a data pin 	      */
#define ICD_CLK  0x01	/* icd2061a clock pin	      */
#define	ICD_NONE 0x00	/* neither DATA nor CLK       */

static void pll_set(struct kgi_display *dpy, ggi_uint clock_cmd)
{
	register ggi_uint i;
	ggi_uint m;

	CLK_out8(dpy, ICD_DATA, 0);	/* do setup as described in [1]	*/
	CLK_out8(dpy, ICD_NONE, 0);
	CLK_out8(dpy, ICD_DATA, 0);

	for (i = 0; i < 6; i++) {

		CLK_out8(dpy, ICD_BOTH, 0);
		CLK_out8(dpy, ICD_DATA, 0);
	}
	for (i = 0; i < 2; i++) {

		CLK_out8(dpy, ICD_NONE, 0);
		CLK_out8(dpy, ICD_CLK, 0);
	}

	for (i = 0, m = 1;  i < 24;  i++, m += m) {

		if (clock_cmd & m) {

			CLK_out8(dpy, ICD_CLK, 0);
			CLK_out8(dpy, ICD_NONE, 0);
			CLK_out8(dpy, ICD_DATA, 0);
			CLK_out8(dpy, ICD_BOTH, 0);

		} else {

			CLK_out8(dpy, ICD_BOTH, 0);
			CLK_out8(dpy, ICD_DATA, 0);
			CLK_out8(dpy, ICD_NONE, 0);
			CLK_out8(dpy, ICD_CLK, 0);
		}
	}

	CLK_out8(dpy, ICD_BOTH, 0);
	CLK_out8(dpy, ICD_DATA, 0);
	CLK_out8(dpy, ICD_BOTH, 0);
	CLK_out8(dpy, ICD_DATA, 0);

	/* usleep(3000);	wait for 3 ms should be enough */
	CLK_out8(dpy, CLOCK_GENERATOR, 0);
}

#include "pll.inc"
