/* ---------------------------------------------------------------------------
 * IBM VGA framebuffer support
 * ---------------------------------------------------------------------------
 *
 * Copyright (C) 1997, The Linux-GGI Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * -----------------------------------------------------------------------
 *
 *   $Log: vga.c,v $
 *   Revision 1.11  1997/12/14 21:56:03  becka
 *   uint/sint namespace fixed ... please test ...
 *
 *   Revision 1.10  1997/09/06 06:16:30  taylorj
 *   *** empty log message ***
 *
 *   Revision 1.9  1997/06/16 20:47:35  seeger
 *   updated kgim_*_done()
 *   minor compile warning fixes
 *
 *   Revision 1.8  1997/04/17 05:08:04  ezrec
 *   Major: 	New ggiOpen() semantics ie ggiOpen("display-KGI:/dev/graph3",NULL)
 *   	First attempt at linear_1 frame buffer
 *   	Moved thread support `to the side'
 *   	Added DPRINT() support
 *   	Added display-KGI driver
 *
 *   Revision 1.7  1997/04/12 04:51:29  ezrec
 *   Minor: Fixes to the VGA subsystem to work w/Steffen's new MMAP system
 *
 *   Revision 1.6  1997/04/10 06:36:38  ezrec
 *   Major: Fixes for Steffen's new MMAP...
 *
 *   Revision 1.5  1997/04/10 03:34:06  ezrec
 *   Major: More fixes, and I'm attempting (ha!) to use Steffen's new MMAP scheme..
 *
 *   Revision 1.4  1997/04/09 04:18:01  ezrec
 *   Major: Added palettes in include/, fixed 8bit for VGA, and
 *          got libggi-dynamic operational... Now to make it work
 *          for the _new_ MMAP scheme!
 *
 *   Revision 1.3  1997/03/26 23:51:00  becka
 *   Better MMAP consistency checking. VGA mmap handler fixed. Still need
 *   correct setpage.
 *
 *   Revision 1.2  1997/03/25 04:54:06  ezrec
 *   Major: libggi now handles libggi.conf files, IBM vga driver suggests
 *         default ioctl RAMDAC DLL, default ioctl RAMDAC DLL support,
 *         -Wall added to the libggi-dynamic Makefiles.
 *
 *   Revision 1.1  1997/03/23 01:26:13  ezrec
 *   Major: Groundwork for libggi-dynamic support...
 *
 * ---------------------------------------------------------------------------
 */

#include <ggi/maintainers.h>
#define	MAINTAINER	Jon_Taylor
#define DRIVER_NAME	"IBM VGA graphics driver"
#define	DRIVER_REV	"$Revision: 1.11 $"

#define DEBUG_LEVEL	0

/* KGI includes */
#include <asm/io.h>

#include <ggi/module.h>
#include <ggi/ggi_commands.h>

#include "suggest.inc"
#include "mmio.inc"
/* KGI module interface */

int kgim_graphics_init(struct kgi_card *card, struct kgi_display *dpy)
{
	/* FIXME: We should really do something here! */
	NOTICE1("VGA Graphics Driver loaded");
	return EOK;
}

void kgim_graphics_done(struct kgi_display *dpy)
{
	NOTICE1("VGA Graphics Driver removed");
}

int kgim_graphics_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
			      enum kgi_tm_cmd cmd)
{
	/* !!! Should we check for accel-friendly dimensions here ??? */
	switch (cmd) {
		case CMD_PROPOSE:
		NOTICE1("Setting up mmio");
		return graphics_mmio_setup(dpy, mode);
		default:
		break;
	}
	
	return EOK;
}

void kgim_graphics_set_mode (struct kgi_display *dpy, struct kgi_mode *mode)
{
}

int kgim_graphics_command (struct kgi_graphic *gr, ggi_uint cmd, ggi_uint arg)
{ 
	ggi_suggest *sugg;
	const char *name;

	switch (cmd) {
		case GRAPHICS_GETSUGGEST:
		sugg = (ggi_suggest *)(gr->io_buf);
		gr->io_res = gr->io_buf;
		
		if (sugg->id < 0 || sugg->id > 1)
			return -E(GRAPHICS, BOGUS);
		if (sugg->id == 1) { /* Generic RAMDAC support */
			strcpy(sugg->name, "generic-ramdac");
			sugg->args[0] = 0;
			sugg->id = 0;
			return 0;
		}
		switch (gr->dev.mode.graphtype) {
			case GT_TEXT16:
			name = sugg_default[SG_TEXT16];
			break;
			case GT_1BIT:
			name = sugg_default[SG_1BIT];
			break;
			case GT_4BIT:
			name = sugg_default[SG_4BIT];
			break;
			case GT_8BIT:
			if (gr->dev.mode.visible.x == 320 &&
			    gr->dev.mode.visible.y == 200)
				name = sugg_default[SG_8BIT];
			else
				name = sugg_default[SG_MODEX];
			break;
			default:
			return -E(GRAPHICS, BOGUS);
		}
		strcpy(sugg->name, name);
		sugg->args[0] = 0;
		sugg->id = 1;	/* Next is RAMDAC */
		return 0;
		default:
		break;
	}
			
	return -E(GRAPHICS, NOSUP_ALWAYS_MMAP);
}

