/* ---------------------------------------------------------------------------
**	KGI kernel driver
** ---------------------------------------------------------------------------
**
**	Copyright (C) 1995-1997 by 
**
**	Andreas Beck,
**	Steffen Seeger,
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
** 
**	$Log: i386.c,v $
**	Revision 1.25  1998/01/03 18:48:21  jonas_b
**	Added dacmhz module parameter.
**	
**	Revision 1.24  1997/12/30 22:09:04  grimrath
**	*** empty log message ***
**	
**	Revision 1.23  1997/12/26 16:48:15  seeger
**	Added 'chipset' string option.
**	
**	Revision 1.22  1997/12/14 21:56:12  becka
**	uint/sint namespace fixed ... please test ...
**
**	Revision 1.21  1997/12/04 10:29:31  seeger
**	Fix in PCI export macros.
**	
**	Revision 1.20  1997/12/03 15:41:54  seeger
**	Added PCI module parameters
**	
**	Revision 1.18  1997/11/08 15:12:47  becka
**	Namespace cleanup ... probably broke all and everything ...
**
** ---------------------------------------------------------------------------
*/

#include <ggi/maintainers.h>
#define	MAINTAINER	Steffen_Seeger
#define DRIVER_NAME	"i386"
#define DRIVER_REV	"$Revision: 1.25 $"

#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/mm.h>

#include <ggi/module.h>
#include <ggi/ggi_commands.h>
#include "config.h"

#include <asm/io.h>

#if DEBUG_LEVEL > 0
static void printk_mode (struct kgi_mode *mode, char *msg)
{
	#define M(X)	mode->X
	#define R(X)	mode->request.X
	#define T(X)	M(X.width), M(X.blankstart), M(X.syncstart),\
			M(X.syncend), M(X.blankend), M(X.total), M(X.polarity)

	printk(msg);
	printk("mode = {\n{\n/* frames    */  %i,\n/* visible */\t{ %i, %i},\n",
			R(frames), R(visible.x),R(visible.y));
	printk("/* virtual   */  { %i, %i},\n",R(virt.x),R(virt.y));
	printk("/* size [mm] */  { %i, %i},\n",R(size.x),R(size.y));
	printk("/* graphmode */  0x%x,\n", R(graphtype));
	printk("/* textgrid  */  { %i, %i}},\n", R(text.x),R(text.y));
	printk("/* x timing  */  { %i, %i, %i, %i, %i, %i, %i},\n", T(x));
	printk("/* y timing  */  { %i, %i, %i, %i, %i, %i, %i},\n", T(y));
	printk("/* magnify   */  { %i, %i},\n", M(magnify.x), M(magnify.y));
	printk("/* flags     */  %x,\n", M(flags));
	printk("/* dclk      */  %i,\n", M(dclk));
	printk("/* clk       */  { %i, %i},\n", M(clk.mul), M(clk.div));
	printk("/* lclk      */  { %i, %i}\n}\n", M(lclk.mul), M(lclk.div));

	#undef M
	#undef R
	#undef T
}
#endif /* #if DEBUG_LEVEL > 0 */

static struct kgi_card card = 
{
	CONFIG_CARD_MANUFACT,			/* Card manufacturer	*/
	CONFIG_CARD_MODEL,			/* Card model		*/
	CONFIG_CARD_MODES,			/* Allowed modes	*/
	0,
	{ CONFIG_CARD_MAX_X, CONFIG_CARD_MAX_Y },/* User limits		*/

	NULL,	/* card component info structs, these are not known yet	*/
	NULL,	/* and get filled in during inititalization		*/
	NULL,
	NULL,

	{	/* The mode to set when resetting the card		*/
		{ 1,				/* need one frame	*/
		  { 720, 400 }, { 720, 400 },	/* visible/virtual	*/
		  { 0, 0 },			/* size in mm		*/
		  GT_TEXT16,			/* mode			*/
		  { 9, 16 }			/* text size		*/
		},

		{ 720, 720, 765, 873, 882, 900,  0 },	/* horiz.timing	*/
		{ 400, 407, 412, 414, 442, 449,  1 },	/* vert. timing	*/
		{ 1, 1 },			/* no magnification	*/
		0,	/* !!! check */		/* special capabilities	*/
		28322000,			/* dot clock in Hz	*/
		{ 1, 1 },			/* clock rate to program*/
		{ 1, 1 },			/* transfer rate to dac	*/

		/* !!! init the reset (to 0 ?) */
        }
};

#define	kgim_set_mode	kgim_chipset_set_mode

static int kgim_check_mode(struct kgi_display *dpy, struct kgi_mode *mode, 
			   enum kgi_tm_cmd cmd)
{
	int cnt = 10;

	if (cmd == CMD_PROPOSE) {

		memset(&(mode->request) + 1, 0, 
		       sizeof(*mode) - sizeof(mode->request));
	}

	while (cnt-- && !(cmd == CMD_READY)) {

		register int err;
		DEBUG1("check_mode:cmd=%i; cnt=%i", cmd, cnt);

		if ((err = kgim_clock_check_mode (dpy, mode, cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: clock ok.");

		if ((err = kgim_ramdac_check_mode(dpy, mode, cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: ramdac ok.");

		if ((err = kgim_chipset_check_mode(dpy, mode, cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: chipset ok.");
 
		if (cmd == CMD_LOWER || cmd == CMD_RAISE) {

			if ((err = kgim_clock_check_mode(dpy, mode, cmd))) 
			{

				return err;
			}
			DEBUG1("2nd clock check ok.");
		}

		if ((err = kgim_graphics_check_mode(dpy, mode, cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: graphics ok.");
 
		if ((err = kgim_monitor_check_mode(dpy, mode, &cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: monitor ok.");

		#if DEBUG_LEVEL > 0
			printk_mode(mode, "mode:\n");
		#endif
	}

	#if DEBUG_LEVEL > 0
	TRACE(printk_mode(mode, "mode ok:\n"));
	#endif
 
	return (++cnt) ? EOK : -E(DRIVER, UNKNOWN);
}

static void kgim_reset(struct kgi_display *dpy)
{
	kgim_chipset_set_mode(dpy, &(dpy->card->preset));

	if (dpy->set_font) {

		struct kgi_font *font;

		font = kgi_default_font(&(dpy->card->preset.request));
		(dpy->set_font)(dpy, font, 0, font->info->positions);
	}

	if (dpy->set_clut) {

		ggi_clut *clut;

		clut = kgi_default_clut(&(dpy->card->preset.request));
		(dpy->set_clut)(dpy, clut, 0, clut->size);
	}

	/*	frame and origin are set to zero and splitline is set to
	**	dpy->card->reset->request.visible.y by the chipset driver
	**	during dpy->set_mode. Thus no need to reset them again.
	*/
}

/*	module loading
*/

ggi_uint pcibus = -1, pcidev = -1, pcifn = 0;
ggi_uint pcibase0 = 0, pcibase1 = 0, pcibase2 = 0, pcibase3 = 0, pcibase4 = 0;
ggi_uint dacmhz = -1;
char	*chipset = NULL;
struct kgim_options_pci options_pci;

ggi_uint law_base = 0, display = 0;

#ifdef MODULE_PARM
#if MAX_NR_DISPLAYS != 32
	#warning MAX_NR_DISPLAYS changed, update this code
#endif
MODULE_PARM(display, "0-" __MODULE_STRING(32) "i");

MODULE_PARM(law_base, "1-" __MODULE_STRING(4294967295) "i");

MODULE_PARM(pcibus, "0-" __MODULE_STRING(255) "i");
MODULE_PARM(pcidev, "0-" __MODULE_STRING(255) "i");
MODULE_PARM(pcibase0, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase1, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase2, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase3, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase4, "1-" __MODULE_STRING(429496295) "i");

MODULE_PARM(dacmhz, "1-" __MODULE_STRING(255) "i");

MODULE_PARM(chipset, "s");
#endif

struct kgim_options_misc options_misc;
struct kgim_options_chipset options_chipset;

static struct kgim_options options =
{
	&options_misc,
	&options_pci,
	&options_chipset
};

static inline void get_options(void)
{
	if ((pcibus != -1) && (pcidev != -1)) {

		options_pci.dev = PCICFG_VADDR(pcibus, pcidev, pcifn);
		options_pci.base0 = pcibase0;
		options_pci.base1 = pcibase1;
		options_pci.base2 = pcibase2;
		options_pci.base3 = pcibase3;
		options_pci.base4 = pcibase4;

	} else {

		memset(&options_pci, 0, sizeof(options_pci));
		options_pci.dev = PCICFG_NULL;
	}

	options_misc.law_base = law_base;
	options_misc.display = display;

	options_chipset.chipset = chipset;
};

static struct kgim_private module_private =
{
	&options,
	NULL,
};

static ggi_mode kgim_default_mode;

struct kgi_display my_display = 
{
	-1,				/* id number			*/
	0,				/* refcnt			*/
	0xFFFFFFFF,			/* conflict_text		*/
	0xFFFFFFFF,			/* conflict_graph		*/
	0xFFFFFFFF,			/* conflict_setup		*/
	NULL,				/* currently mapped device	*/

	&module_private,		/* driver_data			*/

					/* these are set by the driver	*/

	NULL,				/* enable(struct kgi_display *)	*/
	NULL,				/* disable(struct kgi_display *)*/
	kgim_reset,			/* reset(struct kgi_display *)	*/

	{				/* ioctl methods		*/
		NULL,
		kgim_monitor_command,
		kgim_ramdac_command,
		kgim_clock_command,
		kgim_chipset_command,
		kgim_graphics_command,
		NULL,
		NULL
	},
	&card,				/* hardware info		*/
	kgim_check_mode,		/* check_mode()			*/
	kgim_set_mode,			/* set_mode()	-> chipset	*/

	NULL,				/* set_font()	-> chipset	*/
	NULL,				/* set_clut()	-> dac		*/
	NULL,				/* set_split()	-> chipset	*/
	NULL,				/* set_frame()	-> chipset	*/
	NULL,				/* set_origin()	-> chipset	*/
	NULL,				/* set_pointer() -> dac/chipset	*/

	NULL, NULL, NULL,		/* { show, hide, undo }_mouse	*/
	NULL, NULL, NULL,		/* { show, hide, undo }_cursor	*/

	{ },				/* current mode			*/
	&kgim_default_mode,		/* default mode of display	*/
	{ }				/* framebuffer of current frame	*/
};

int init_module(void)
{
	/*	NOTE: The various ?_init() functions have to printk the reason
	**	for failing themself -- just in case they fail.
	*/
	int err;

	get_options();

	/*	The chipset provides I/O functions for all other parts, thus we
	**	have to init it first.
	*/
	if ((err = kgim_chipset_init(&card, &my_display))) {

		return err;
	}

	/*	Now that the chipset is ok, we try to init the monitor driver.
	**	if this fails, we have to deinitializee the chipset.
	*/
	if ((err = kgim_monitor_init(&card, &my_display))) {

		kgim_chipset_done(&my_display);
		return err;
	}

	/*	Now we go for the DAC. If it fails, we have to deinitialize
	**	the subsystems already fired up.
	*/
	if ((err = kgim_ramdac_init(&card, &my_display))) {

		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	Now the clock driver. If this fails, ...
	*/
	if ((err = kgim_clock_init(&card, &my_display))) {

		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	Last is the accel driver. If this fails, ...
	*/
	if ((err = kgim_graphics_init(&card, &my_display))) {

		kgim_clock_done(&my_display);
		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	We require that the card can be reset properly, thus
	**	the preset mode must be accepted by the drivers.
	**	Just in case ... (Ever heard of Murphys Law? |-)
	*/
	if (kgim_check_mode(&my_display, &card.preset, CMD_CHECK)) {
	
		ERROR("check_mode() failed with preset mode.");
		kgim_graphics_done(&my_display);
		kgim_clock_done(&my_display);
		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	Now set the initial display mode and other state. We know
	**	the mode check can't fail because if it does, we should never
	**	have gotten here.
	*/
	memcpy(my_display.defmode, &card.preset.request, sizeof(ggi_mode));
	memcpy(&my_display.mode.request, my_display.defmode, sizeof(ggi_mode));
	kgim_check_mode(&my_display, &my_display.mode, CMD_PROPOSE);
	kgim_chipset_set_mode(&my_display, &my_display.mode);
	kgim_reset(&my_display);

	/*	Ok. Hardware is up and running now, so we can try to register
	**	it. <display> holds the boot display to replace or -1 if the
	**	display has not been registered. If registration fails, we
	**	reset the hardware and shutdown properly.
	*/
	if ((err = kgi_register_display(&my_display, display))) {

		ERROR("failed to register as display %i", display);
		kgim_reset(&my_display);

		kgim_graphics_done(&my_display);
		kgim_clock_done(&my_display);
		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	And now.... FLY BABY, FLY !
	*/
	MOD_INC_USE_COUNT;
	DEBUG1("%s %s driver loaded as display %i.",
	       card.manufact, card.model, my_display.id);
	return EOK;
}

void cleanup_module (void)
{
	if (MOD_IN_USE) {

		DEBUG1("Module use count is not 0.");
	}

	kgim_reset(&my_display);

	kgim_graphics_done(&my_display);
	kgim_clock_done(&my_display);
	kgim_ramdac_done(&my_display);
	kgim_monitor_done(&my_display);
	kgim_chipset_done(&my_display);

	DEBUG1("%s %s driver removed.", card.manufact, card.model);
}
