/* ---------------------------------------------------------------------------
**	NONE ramdac driver
** ---------------------------------------------------------------------------
**
**	Copyright (C) 1995 by Jochen Hoenicke
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
*/

#include <ggi/maintainers.h>
#define	MAINTAINER	Jochen_Hoenicke
#define	DRIVER_NAME	"dummy DAC driver"
#define	DRIVER_REV	"$Revision: 1.3 $"
#define	DEBUG_LEVEL	2

#include <ggi/kgi.h>
#include <ggi/module.h>

static struct kgi_ramdac ramdac = {

	"NONE",				/* manufacturer		*/
 	"NONE",				/* model		*/
	GM_ALL,				/* supported modes	*/
 	0,			 	/* flags		*/
 	{ 0xFFFF, 0xFFFF },		/* maximum resolution	*/

	0,				/* device type		*/
	  {       0,  32000000 },	/* normal mode		*/
	  {       0,  32000000 },	/* 2xclk mode		*/

	 {{	  0,  32000000 },	/* 	GT_TEXT16	*/
	  {	  0,  32000000 },	/* 	GT_TEXT32 	*/
	  {       0,  32000000 },	/* 	GT_1BIT		*/
	  {	  0,  32000000 },	/* 	GT_4BIT		*/
	  {       0,  32000000 },	/* 	GT_8BIT		*/	
	  {	  0,  32000000 },	/* 	GT_15BIT	*/
	  { 	  0,  32000000 },	/* 	GT_16BIT	*/
	  {       0,  32000000 },	/* 	GT_24BIT	*/
	  {	  0,  32000000 }}	/* 	GT_32BIT	*/
};

/*
**	GGI interface
*/

int kgim_ramdac_init(struct kgi_card *card,struct kgi_display *dpy)
{
        NOTICE(DRIVER_NAME " rev " DRIVER_REV);
        card->ramdac = &ramdac;
        return EOK;
}

void kgim_ramdac_done(struct kgi_display *dpy)
{
       NOTICE(DRIVER_NAME " removed.");
}

int kgim_ramdac_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
	enum kgi_tm_cmd cmd)
{
        switch (cmd) {

        case CMD_PROPOSE:
                mode->lclk.mul = mode->clk.mul = 1;
                mode->lclk.div = mode->clk.div = 1;
                return EOK;
                
        case CMD_LOWER:
        case CMD_RAISE:
                return EOK;

        case CMD_CHECK:
                if ((mode->lclk.mul != 1) || (mode->lclk.div != 1) ||
                    (mode->clk.mul  != 1) || (mode->clk.div  != 1)) {

                        return -E(RAMDAC,MODE_CHECK);
		}
                
                return EOK;
                
        default:
                INTERNALERROR;
                return -E(RAMDAC,BOGUS);
        }
}

void kgim_ramdac_set_mode(struct kgi_display *display,struct kgi_mode *mode)
{
}

int kgim_ramdac_ioctl (struct inode *inode, struct file *file,
	ggi_uint cmd, ggi_uint arg)
{
        /* This driver can't do this. */
        return -E(RAMDAC,NOSUP_ALWAYS_CANT);
}

