/* 
   Graphics library for GGI. Mode management.

   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dlfcn.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <signal.h>

#include "ggi-dl.h"
#include "internal.h"
#include "aavisual.h"

#include <ggi/ggi_commands.h>

extern int        _GGIbpc;
extern int        _GGInocols;
extern ggi_color *_GGIcolormap;

int GGIsetfocus(ggi_visual *vis)
{
	return EOK;
}

int GGIsetorigin(ggi_visual *vis,int x,int y)
{
	return E(DRIVER,NOSUP_ALWAYS_CANT);
}

int GGIsetsplitline(ggi_visual *vis,int y)
{
	return E(DRIVER,NOSUP_ALWAYS_CANT);
}

static int _GGIdomode(ggi_visual *vis)
{
	int err;
	ggi_suggest sug;

	char sugs[1][80];
	
	switch(VIS_MODE(vis)->graphtype)
	{
	  case GT_8BIT : err=8;break;
	  default: return -EINVAL;
	}

	sprintf(sugs[0],"generic-linear-%d",err);

	_ggiZapMode(vis,~GGI_DL_OPDISPLAY);

	_ggiSetupMode(vis);

	err=(_ggiOpenDL(vis,"generic-stubs","")==NULL);
	if (err) {
		fprintf(stderr,"display-AA: Can't open the \"generic-stubs\" "
				"library.\n");
	}

	sug.id=0;
	do {
		/* FIXME ! */
		strcpy(sug.name,sugs[sug.id]);
		strcpy(sug.args,"");

		err=(_ggiOpenDL(vis,sug.name,sug.args)==NULL);
		if (err) {
			fprintf(stderr,"display-X: Can't find an appropriate "
				       "library for %s (%s)\n",
					sug.name,sug.args);
			return err;
		} else {
			DPRINT("Success in loading %s (%s)\n",sug.name,sug.args);
		}
	} while (sug.id!=0);

	/* Override the color operations for X if we are 8 bit palette. */
	if (_GGIcolormap) { 
		free(_GGIcolormap);_GGIcolormap=NULL;
	}

	if (VIS_MODE(vis)->graphtype==GT_8BIT) {

		int x;
	        _GGIbpc=1;
		_GGInocols=256;
		_GGIcolormap=malloc(256*sizeof(ggi_color));
		for(x=0;x<_GGInocols;x++)
		{
			_GGIcolormap[x].r=((x&0xe0)>>5)*0x2492;
			_GGIcolormap[x].g=((x&0x1c)>>2)*0x2492;
			_GGIcolormap[x].b=((x&0x03)   )*0x5555;
		} 
		vis->opcolor->mapcolor=GGImapcolor;
		vis->opcolor->unmappixel=GGIunmappixel;
		vis->opcolor->setpalvec=GGIsetpalvec;
		vis->opcolor->getpalvec=GGIgetpalvec;
		/* This is just to trigger the initial setup. */
		GGIsetpalvec(vis,0,256,_GGIcolormap);
	}

	return EOK;
}

int GGIdoflush(struct AAhooks *aahook)
{
	static int lock=0;
	
	if (lock++!=0) {lock--;return -EAGAIN;}	/* FXIME use spinlocks here !*/

	aa_renderpalette(aahook->context,aahook->pal, &aa_defrenderparams, 0, 0, 
			aa_scrwidth(aahook->context), aa_scrheight(aahook->context));
	aa_flush(aahook->context);

	lock--;	/* FIXME Spinlocks ... */

	return EOK;
}

static struct AAhooks *ahk;

void _doflush_ALRM(int unused)
{ 
	struct itimerval tval={{0,00000},{0,200000}};
	signal(SIGALRM,_doflush_ALRM);
	GGIdoflush(ahk);
	setitimer(ITIMER_REAL,&tval,NULL);
}

int GGIflush(ggi_visual *vis)
{
	return GGIdoflush((struct AAhooks *)VIS_PRIVATE(vis));
}

int GGIsetmode(ggi_visual *vis,ggi_mode *tm)
{ 
	struct AAhooks *aahook=VIS_PRIVATE(vis);

	if (vis==NULL) {
		DPRINT("Visual==NULL\n");
		return -1;
	}
	
	if (VIS_FB_LINEAR(vis)!=NULL) {
		VIS_FB_LINEAR(vis)=NULL;
		VIS_FB_LINEAR_SIZE(vis)=0;
	}

	switch(tm->graphtype) { 
	    case GT_8BIT : 
	    	aahook->context=aa_autoinit(&aa_defparams);
		tm->virt.x=aa_imgwidth (aahook->context);
		tm->virt.y=aa_imgheight(aahook->context);
		VIS_FB_LINEAR(vis)=aa_image(aahook->context);
		VIS_FB_LINEAR_SIZE(vis)=tm->virt.x*tm->virt.y;
		aa_autoinitkbd(aahook->context,AA_SENDRELEASE);
		aa_autoinitmouse(aahook->context,AA_MOUSEALLMASK);
		break;
	    default:
		fprintf(stderr,"Sorry. The aalib target only supports 8 bit palette operation.\n");
	  	return GGI_DL_ERROR;
	}

	GGIdoflush(ahk=aahook);
	signal(SIGALRM,_doflush_ALRM);
	alarm(1);
	memcpy(VIS_MODE(vis),tm,sizeof(ggi_mode));

	return _GGIdomode(vis);
}

/**********************************/
/* check any mode (text/graphics) */
/**********************************/
int GGIcheckmode(ggi_visual *vis,ggi_mode *tm)
{
	if (vis==NULL)
		return -1;
	
	return 0; /* FIXME ! */
}

/************************/
/* get the current mode */
/************************/
int GGIgetmode(ggi_visual *vis,ggi_mode *tm)
{
	DPRINT("In GGIgetmode(%p,%p)\n",vis,tm);
	if (vis==NULL)
		return -1;
	
	/* We assume the mode in the visual to be o.k. */
	memcpy(tm,VIS_MODE(vis),sizeof(ggi_mode));

	return 0;
}
