/* 
 *  Graphics library for GGI. Mode management.
 *
 *  Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu
 *
 *  If you do any modifications, I would like you to send diffs to me
 *  to allow for collecting a more and more complete set of drivers and
 *  to improve existing ones.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ---------------------------------------------------------------------
 *	
 *	$Log: stubs.c,v $
 *	Revision 1.9  1997/12/18 20:39:49  becka
 *	Updated a few things ...
 *	
 *	Revision 1.8  1997/12/14 21:56:35  becka
 *	uint/sint namespace fixed ... please test ...
 *	
 *	Revision 1.7  1997/12/07 16:27:57  becka
 *	LibGGI extension fixes
 *	
 *	Revision 1.6  1997/11/15 01:13:20  willie
 *	Changed GPL to LGPL License
 *	
 *	Revision 1.5  1997/11/03 15:12:01  becka
 *	added keyboard.h, made ASYNC work.
 *	
 *	Revision 1.4  1997/10/26 23:46:22  becka
 *	Restructured default visuals.
 *	Added some X keysyms
 *	Added some cards
 *	
 *	Revision 1.3  1997/10/17 22:24:41  becka
 *	Preliminary DirectBuffer support.
 *	
 *	Revision 1.2  1997/10/15 22:09:30  becka
 *	Added ggiCrossBlit. FIX and OPTIMIZE this.
 *	
 *	Revision 1.1  1997/10/12 04:53:42  tfries
 *	new subdir
 *	
 *	Revision 1.7  1997/07/14 06:28:14  ezrec
 *	Major: Revamped a lot of stuff, all old binaries no longer work,
 *	       added ggiGetInfo, ggiLoadLibrary, ggiGetLibraryFunc,
 *	       `macroized' access to the ggi_visual struct for libraries.
 *	
 *	Revision 1.6  1997/06/24 14:15:56  ezrec
 *	Minor: fixed setsplit <-> setsplitline bugaboo
 *
 *	Revision 1.5  1997/06/23 05:07:55  ezrec
 *	Major:  Moved all the `device independant' functions (bresenham draw line,
 *		circle, etc) to a `stubs' lib. Also, created a more `kernel esque'
 *		demo/.evstack that uses libGGI for all I/O.
 *
 *	Revision 1.4  1997/05/16 05:12:13  ezrec
 *	Major: Event support, and updated ggilib shim. Xggi 0.0.8 binaries should
 *		work now, with a minor caveat on mouse buttons..
 *
 *	Revision 1.3  1997/04/17 05:08:13  ezrec
 *	Major: 	New ggiOpen() semantics ie ggiOpen("display-KGI:/dev/graph3",NULL)
 *		First attempt at linear_1 frame buffer
 *		Moved thread support `to the side'
 *		Added DPRINT() support
 *		Added display-KGI driver
 *
 *	Revision 1.2  1997/03/25 04:54:15  ezrec
 *	Major: libggi now handles libggi.conf files, IBM vga driver suggests
 *	      default ioctl RAMDAC DLL, default ioctl RAMDAC DLL support,
 *	      -Wall added to the libggi-dynamic Makefiles.
 *
 *	Revision 1.1  1997/03/23 08:27:40  ezrec
 *	Major: libggi-dynamic now uses `stubs' instead of #defines so that
 *	       new versions of the library won't break old ones...
 *
 *
 * ---------------------------------------------------------------------
 */
#include <string.h>
#include "internal.h"

#undef putc

/************** Mode info ****************/

const ggi_info *ggiGetInfo(ggi_visual *vis)
{ return &vis->info; }

int ggiSetInfoFlags(ggi_visual *vis,ggi_uint flags)
{  VIS_FLAGS(vis)=flags; 
   /* BUGBUG - we should do something.. */
   return EOK;
}

/*************** Lib management ***********************/

ggi_lib_id ggiLoadLibrary(ggi_visual *vis,const char *name,const char *args)
{ return (ggi_lib_id)_ggiAddDL(vis,name,args,GGI_DLTYPE_EXTENSION); }

ggi_lib_func ggiGetLibraryFunc(ggi_visual *vis,ggi_lib_id id,ggi_uint func)
{ if (((ggi_dlhandle *)id)->privfunc == NULL)
	return NULL;
   else return (ggi_lib_func)((ggi_dlhandle *)id)->privfunc(vis,func); }

int ggiUnloadLibrary(ggi_visual *vis,ggi_lib_id lib)
{ if (lib) {lib->use=0;_ggiRemoveDL(vis,&vis->extlib);} return EOK;}

ggi_lib_id ggiFindLibrary(ggi_visual *vis,const char * name)
{
	ggi_dlhandle_l *tmp=vis->extlib;
	while (tmp && strcmp(tmp->handle->name,name)) tmp=tmp->next; 
	if (tmp) return (ggi_lib_id) tmp->handle; else return NULL;	
}

/*************** Display management *******************/

/* Flush _all_ pending accel commands, and ready the visual for software 
 * drawing
 */
int ggiFlush(ggi_visual *vis)
{  return vis->opdisplay->flush(vis); }

/* Palette, colors, etc...
 */
int ggiSetPaletteVec(ggi_visual *vis,int s,int len,ggi_color *cmap)
{ return vis->opcolor->setpalvec(vis,s,len,cmap); }

int ggiGetPaletteVec(ggi_visual *vis,int s,int len,ggi_color *cmap)
{ return vis->opcolor->getpalvec(vis,s,len,cmap); }

ggi_uint ggiMapColor(ggi_visual *vis,ggi_color col)
{ return vis->opcolor->mapcolor(vis,col); }

int ggiUnmapPixel(ggi_visual *vis,ggi_uint pixel,ggi_color *col)
{ return vis->opcolor->unmappixel(vis,pixel,col); }

int ggiPackColors(ggi_visual *vis,void *buf,ggi_color *cols,int len)
{ return vis->opcolor->packcolors(vis,buf,cols,len); }

int ggiUnpackPixels(ggi_visual *vis,void *buf,ggi_color *cols,int len)
{ return vis->opcolor->unpackpixels(vis,buf,cols,len); }

/* Gamma mapping ...
 */

int  ggiGetGamma(ggi_visual_t vis,ggi_float *r,ggi_float *g,ggi_float *b)
{ return vis->opcolor->getgamma(vis,r,g,b); }

int  ggiSetGamma(ggi_visual_t vis,ggi_float  r,ggi_float  g,ggi_float  b)
{ return vis->opcolor->setgamma(vis,r,g,b); }

int  ggiGetGammaMap(ggi_visual_t vis,int s,int len,ggi_color *gammamap)
{ return vis->opcolor->getgammamap(vis,s,len,gammamap); }

int  ggiSetGammaMap(ggi_visual_t vis,int s,int len,ggi_color *gammamap)
{ return vis->opcolor->setgammamap(vis,s,len,gammamap); }

/* Origin, Splitline, etc...
 */
int ggiSetOrigin(ggi_visual *vis,int x,int y)
{ return vis->op2d->setorigin(vis,x,y); }

int ggiSetSplitline(ggi_visual *vis,int y)
{ return vis->op2d->setsplitline(vis,y); }



/* Graphics sprite manipulation
 */
void *ggiNewSprite(ggi_visual *vis,int w,int h,void *image)
{ return vis->op2d->newsprite(vis,w,h,image); }

int ggiSetSprite(ggi_visual *vis,void *pid,int v,int x,int y)
{ return vis->op2d->setsprite(vis,pid,v,x,y); }

int ggiFreeSprite(ggi_visual *vis,void *pid)
{ return vis->op2d->freesprite(vis,pid); }




/* Generic drawing routines 
 */
int ggiPutc(ggi_visual *vis,int x,int y,char c)
{ return vis->op2d->putc(vis,x,y,c); }

int ggiPuts(ggi_visual *vis,int x,int y,const char *str)
{ return vis->op2d->puts(vis,x,y,str); }



int ggiFillscreen(ggi_visual *vis)
{ return vis->op2d->fillscreen(vis); }



int ggiDrawPixel(ggi_visual *vis,int x,int y)
{ return vis->op2d->drawpixel(vis,x,y); }

int ggiPutPixel(ggi_visual *vis,int x,int y,ggi_uint col)
{ return vis->op2d->putpixel(vis,x,y,col); }

int ggiGetPixel(ggi_visual *vis,int x,int y,ggi_uint *col)
{ return vis->op2d->getpixel(vis,x,y,col); }



int ggiDrawLine(ggi_visual *vis,int x,int y,int xe,int ye)
{ return vis->op2d->drawline(vis,x,y,xe,ye); }

int ggiDrawHLine(ggi_visual *vis,int x,int y,int w)
{ return vis->op2d->drawhline(vis,x,y,w); }

int ggiPutHLine(ggi_visual *vis,int x,int y,int w,void *buf)
{ return vis->op2d->puthline(vis,x,y,w,buf); }

int ggiGetHLine(ggi_visual *vis,int x,int y,int w,void *buf)
{ return vis->op2d->gethline(vis,x,y,w,buf); }



int ggiDrawVLine(ggi_visual *vis,int x,int y,int h)
{ return vis->op2d->drawvline(vis,x,y,h); }

int ggiPutVLine(ggi_visual *vis,int x,int y,int h,void *buf)
{ return vis->op2d->putvline(vis,x,y,h,buf); }

int ggiGetVLine(ggi_visual *vis,int x,int y,int h,void *buf)
{ return vis->op2d->getvline(vis,x,y,h,buf); }



int ggiDrawBox(ggi_visual *vis,int x,int y,int w,int h)
{ return vis->op2d->drawbox(vis,x,y,w,h); }

int ggiPutBox(ggi_visual *vis,int x,int y,int w,int h,void *buf)
{ return vis->op2d->putbox(vis,x,y,w,h,buf); }

int ggiGetBox(ggi_visual *vis,int x,int y,int w,int h,void *buf)
{ return vis->op2d->getbox(vis,x,y,w,h,buf); }



int ggiCopyBox(ggi_visual *vis,int x,int y,int w,int h,int nx,int ny)
{ return vis->op2d->copybox(vis,x,y,w,h,nx,ny); }

int ggiCrossBlit(ggi_visual *src,int sx,int sy,int sw,int sh,
		 ggi_visual *dst,int dx,int dy,int dw,int dh)
{ /* Note: We use dst to map the request to, as it will normally be
     the "smarter" device. */
  return dst->op2d->crossblit(src,sx,sy,sw,sh,dst,dx,dy,dw,dh); }

int ggiDrawCircle(ggi_visual *vis,int x,int y,int r)
{ return vis->op2d->drawcircle(vis,x,y,r); }



