/* 
   Graphics library for GGI.

   Copyright (C) 1995 Andreas Beck - becka@hp.rz.uni-duesseldorf.de
   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <string.h>

#include "ggi-dl.h"
#include "modex.h"

/* GGIdrawbox
 *  - Worst case: 3 page faults (non int-aligned hlines)
 */
int GGIdrawbox(ggi_visual *vis,int x,int y,int w,int h)
{
	unsigned char color,*pixbuff,*ptr;
	int i,j,row,mask,sy,sw;

	CHECKXYWH(vis,x,y,w,h);

	color=VIS_GC_FGCOLOR(vis);

	pixbuff=(unsigned char *)__localfb;
	sw=VIS_FB_WIDTH(vis)>>2; sy=(y*sw);

	/* Draw the `front' pixels */
	if ((i=x&3)) {
		if ((j=w+i-4)<=0) {
			mask=(0xf<<i)&(0xf>>-j);
			ptr=pixbuff+(x>>2)+sy+(mask<<16);
			for (row=h;row--;ptr+=sw) *ptr=color;
			return EOK;
		} else {
			mask=(0xf<<i)&0xf;
			ptr=pixbuff+(x>>2)+sy+(mask<<16);
			for (row=h;row--;ptr+=sw) *ptr=color;
			w=w+i-4;x=x-i+4;
		}
	}

	/* Draw the "center"  pixels*/
	if (w>>2) {		
		ptr=pixbuff+(x>>2)+sy+0xF0000;
		for (row=h;row--;ptr+=sw)
			memset(ptr,color,w>>2);
	}

	/* Draw the `back' pixels */
	if ((i=(x+w)&3)) {
		mask = (1<<i)-1;

		ptr=pixbuff+((x+w)>>2)+sy+(mask<<16);
		for (row=h;row--;ptr+=sw)
			*ptr=color;
	}

	return EOK;
}

/* GGIputbox
 * Worst case: 4 page faults (boxes > 3 pixels)
 */
int GGIputbox(ggi_visual *vis,int x,int y,int w,int h,void *buffer)
{ 
	unsigned char *ptr,*pixbuff,*cp;
	int sl,i,j,bw,row;

	CHECKXYWH(vis,x,y,w,h);

	pixbuff=(unsigned char *)__localfb;
	bw=w;

	sl=(x>>2)+(VIS_FB_WIDTH(vis)>>2)*y;
	for (i=0;i<4 && w>0;i++,w--) {
		cp=(unsigned char *)buffer+i;
		ptr=pixbuff+(0x10000<<(x&3))+sl;
		for (row=h;row--;cp+=bw) {
			for (j=0;j<w;j+=4) 
				*(ptr++)=cp[j];
			ptr+=(VIS_FB_WIDTH(vis)>>2)-((w+3)>>2);
		}
		x++;
		if ((x&3) == 0) 
			sl++;
	}
	
	return 0;
}

/* GGIgetbox
 * Worst case: 4 page faults (boxes > 3 pixels)
 */
int GGIgetbox(ggi_visual *vis,int x,int y,int w,int h,void *buffer)
{ 
	unsigned char *ptr,*pixbuff,*cp;
	int sl,i,j,bw,row;

	CHECKXYWH(vis,x,y,w,h);

	pixbuff=(unsigned char *)__localfb;
	bw=w;

	sl=(x>>2)+(VIS_FB_WIDTH(vis)>>2)*y;
	for (i=0;i<4 && w>0;i++,w--) {
		cp=(unsigned char *)buffer+i;
		ptr=pixbuff+((x&3)<<16)+sl;
		for (row=h;row--;cp+=bw) {
			for (j=0;j<w;j+=4) 
				cp[j]=*(ptr++);
			ptr+=(VIS_FB_WIDTH(vis)>>2)-((w+3)>>2);
		}
		x++;
		if ((x&3) == 0) 
			sl++;
	}
	
	return 0;
}
