/* 
   Mystique userspace acceleration library

   Copyright (C) 1997 Matthias Grimrath - m.grimrath@tu-bs.de

   Anyone is welcome to improve this lib and send diffs to me.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>

#include <Matrox/mga_common_lib.h>
#include <Matrox/mga_common_regs.h>

#include "ggi-dl.h"
#include "mystique-8-15-16-32.h"

int GGIdrawbox(ggi_visual *vis, int x, int y, int w, int h)
{
	uint32 *db = (uint32 *)__dmawin;

	DPRINT("drawbox: fg=0x%.8x, x=%d, y=%d, width=%d, height=%d, "
	       "dwgctl=%.8x\n",VIS_GC_FGCOLOR(vis),x,y,w,h,
	       TRAP | __blkmode | SOLID | ARZERO | SGNZERO | SHFTZERO |
	       BOP_COPY | BLTMOD_BMONOLEF);
	
	*db++ = ((DMA_FCOL | DMA_GO) << 24) | (DMA_FXBNDRY << 16) |
	        (DMA_YDSTLEN << 8) | (DMA_DWGCTL << 0);
	*db++ = TRAP | __blkmode | SOLID | ARZERO | SGNZERO | SHFTZERO |
	        BOP_COPY | BLTMOD_BMONOLEF;
	*db++ = (y << 16) | h;
	*db++ = ((x+w) << 16) | x;
	*db++ = VIS_GC_FGCOLOR(vis);

	return ioctl(VIS_FD(vis),GR_MATROX_DMASTOP_AND_WAITACCEL);
}

/*
** This small ASCII chart is supposed to help you understand that
** wild "if" block. The origin is (x,y), i.e. the top left corner of
** the source.
** 
**           top-down
**              |
** left-right---+---right-left
**              |
**          bottom-up
*/
int GGIcopybox(ggi_visual *vis, int x, int y, int w, int h, int nx, int ny)
{
	ggi_uint lefttop;
	uint32 *db = (uint32 *)__dmawin;

	DPRINT("copybox: x=%d, y=%d, w=%d, h=%d nx=%d ny=%d\n",x,y,w,h,nx,ny);
	
	/* Checking values is for cowards ... ;) */
	*db++ = (DMA_SGN << 24) | (DMA_AR0 << 16) |
	        (DMA_AR3 << 8) | (DMA_AR5 << 0);
	if (ny < y) { /* top-down */

		*db++   = (VIS_DB(vis)->buffer.pixellinearbuffer.size.x);
		lefttop = (y * VIS_DB(vis)->
			   buffer.pixellinearbuffer.size.x) + x; 
		if (nx < x) { /* left-right */
			*db++ = lefttop;
			*db++ = lefttop + w - 1;
			*db++ = 0; 
		} else { /* right-left */
			*db++ = lefttop + w - 1;
			*db++ = lefttop;
			*db++ = SGN_SCANLEFT; 
       		}
		*db++ = ((DMA_DWGCTL | DMA_GO) << 16) | 
		        (DMA_FXBNDRY << 8) | (DMA_YDSTLEN << 0);
		*db++ = (ny << 16) | h;
		
	} else { /* bottom-up */
		
		*db++   = -(VIS_DB(vis)->buffer.pixellinearbuffer.size.x);
		lefttop = ((y+h-1) * VIS_DB(vis)->
			   buffer.pixellinearbuffer.size.x) + x; 
		if (nx < x) { /* left-right */
			*db++ = lefttop;
			*db++ = lefttop + w - 1;
			*db++ = SGN_SDY; 
		} else { /* right-left */
			*db++ = lefttop + w - 1;
			*db++ = lefttop;
			*db++ = SGN_SDY | SGN_SCANLEFT;
		}
		*db++ = ((DMA_DWGCTL | DMA_GO) << 16) | (DMA_FXBNDRY << 8) |
		        (DMA_YDSTLEN << 0);
		*db++ = ((ny+h-1) << 16) | h;
	}
	*db++ = ((nx + w - 1) << 16) | nx;
	*db++ = BITBLT | ATYPE_RPL | SHFTZERO | BOP_COPY | BLTMOD_BFCOL;

	return ioctl(VIS_FD(vis),GR_MATROX_DMASTOP_AND_WAITACCEL);
}
