/* 
   Mystique userspace acceleration library

   Copyright (C) 1997 Matthias Grimrath - m.grimrath@tu-bs.de

   Anyone is welcome to improve this lib and send diffs to me.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#include "ggi-dl.h"
#include <ggi/ggi_commands.h>
#include <Matrox/mga_common_lib.h>
#include <Matrox/mga_common_regs.h>
#include <Matrox/mga1064_main.h>

#include "mystique-8-15-16-32.h"

/* 
 * We are guaranteed to have been checked already...
 */

unsigned char *__dmawin=NULL;
ggi_graphtype __gtype;
int __blkmode;
int __bpp;
ggi_common_plb_setup __plb_setup;
const char *__dbname;
ggi_uint (*__fallback_mapcolor)(ggi_visual  *, ggi_color);

static int __dmawin_len;
static ggi_gc *_ggiGC=NULL;

/*
** GGIlib interface
*/

int GGIdlinit(ggi_visual *visual,const char *version)
{
	struct myst_info info;
	__dmawin_len=0x1000;

#ifdef ACCELERATION
	__dmawin=mmap(NULL,
		   __dmawin_len, /* 4k == 1 page */
		   PROT_WRITE,
		   MAP_SHARED,
		   VIS_FD(visual),
		   MMAP_TYPE_MMIO|MMAP_PER_REGION_TYPE|MMAP_DMAWIN);
	if (__dmawin==NULL ||
	    (int)(__dmawin) == -1) 
		return GGI_DL_ERROR;
#endif
	if (VIS_GC(visual)==NULL) {
		_ggiGC=malloc(sizeof(ggi_gc));
		if (_ggiGC==NULL) {
			if (__dmawin!=NULL)
				free(__dmawin);
			return GGI_DL_ERROR;
		}
		_ggiGC->version=0;
/*		_ggiLockInit(_ggiGC);*/
	}

	if (VIS_GC(visual)==NULL)
		VIS_GC(visual)=_ggiGC;

	__gtype = visual->info.mode->graphtype;
	ioctl(VIS_FD(visual),CHIP_MYST_GETINFO, &info); /* SD- or SGRAM? */
	__blkmode = info.blkmode; 
	DPRINT("blockmode=%08x\n", __blkmode);
	
	/* Positioning is not supported...
	 */

	/* Sprites aren't supported...
	 */

	/* Generic drawing
	 */
#ifdef ACCELERATION	
	visual->op2d->fillscreen=GGIfillscreen;
	visual->op2d->drawbox=GGIdrawbox;
	visual->op2d->copybox=GGIcopybox;
	visual->op2d->drawhline=GGIdrawhline;
	visual->op2d->drawvline=GGIdrawvline;
	visual->op2d->drawline=GGIdrawline;
#endif
	/* Directbuffer setup */
	/* ??? unmap color also? */
	__fallback_mapcolor=visual->opcolor->mapcolor;
	switch(__gtype) {
	case GT_8BIT:  
		__bpp=8;  
		__plb_setup=sp8a8i8;
		__dbname="I";
		visual->opcolor->mapcolor=GGImapcolor8;
		break;
	case GT_15BIT: 
		__bpp=15; 
		__plb_setup=sp16a16r5g5b5A1; 
		__dbname="RGB";
		visual->opcolor->mapcolor=GGImapcolor16;
		break;
	case GT_16BIT: 
		__bpp=16; 
		__plb_setup=sp16a16r5g6b5;   
		__dbname="RGB";
		visual->opcolor->mapcolor=GGImapcolor16;
		break;
	case GT_24BIT: 
		__bpp=24; 
		__plb_setup=sp24a8b8g8r8;    
		__dbname="RGB";
		break;
	case GT_32BIT: 
		__bpp=32; 
		__plb_setup=sp32a32b8g8r8A8;
		__dbname="RGB";
		break;
	default: 
		ggiPanic("Unsupp. bpptype!");
	};
       	
	GGIdbinit(visual);
	
	return GGI_DL_OPCOLOR|GGI_DL_OP2D;
}

int GGIdlcleanup(ggi_visual *visual)
{
	if (__dmawin!=NULL) {
		munmap(__dmawin,__dmawin_len);
	}

	if (VIS_GC(visual)==_ggiGC) {
/*		_ggiLockDestroy(VIS_GC(visual));*/
		free(VIS_GC(visual));
		VIS_GC(visual)=NULL;
		_ggiGC=NULL;
	}

	return EOK;
}
