/* Keyboard library functions */
/* H. Hanemaayer (hhanemaa@cs.ruu.nl) */

/*
 * Keyboard I/O based on showkey.c from kbd-0.84 package.
 *
 * This is an initial version, it isn't very safe yet since it only catches
 * sigsegv.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/signal.h>

#include "internal.h"
#include "vga.h"
#include "vgakeyboard.h"

/* Workaround. my .h files don't have this. */
#ifndef DONT_CATCH_CTRLC
#define DONT_CATCH_CTRLC 0x80
#endif

static void (*__keyboard_eventhandler) (int, int);


static int ctrl_c_state_c, ctrl_c_state_ctrl;
static int translatemode = 0;
static unsigned char state[NR_KEYS];	/* NR_KEYS is defined in linux/keyboard.h */

static void default_handler(int, int);

int keyboard_init_return_fd(void)
{
    /* Install default keyboard handler. */
    __keyboard_eventhandler = default_handler;

    if (IN_GGI < -1)
	vga_init();

    return EVENT_FD;	/* OK, return fd. */
}

/* Old compatible init function. */

int keyboard_init(void)
{
    if (keyboard_init_return_fd() == -1)
	return -1;
    else
	return 0;
}

void keyboard_close(void)
{
}

/* For now, we assume there's no console switching. */

static int keyboard_getevents(int wait)
{
/* Read keyboard device, and handle events. 
 * If wait == 1, process at least one event and return. 
 * If wait == 0, handle all accumulated events; return 0 if no events 
 * 		 were handled, 1 otherwise. 
 */
    int bytesread, i;
    int eventhandled;
    int scancode;
    int type;
    ggi_event ev;
    ggi_event_mask mask;
    struct timeval *tp,ztime={0,0};

    eventhandled = 0;

    if (wait) 
	tp=NULL;
    else
	tp=&ztime;

    mask=ggiEventPoll(MASTER_VIS,emKeyboard,tp);
    if (!(mask & emKeyboard)) {
	return eventhandled;
    }

    do {
	ggiEventRead(MASTER_VIS,&ev,emKeyboard);

	type=ev.any.type;
	scancode=ev.key.code;	

	/* Check for ctrl-c. */
	if (scancode == SCANCODE_C)
	    if (type==evKeyRelease)
		ctrl_c_state_c = 0;
	    else
		ctrl_c_state_c = 1;

	if (scancode == SCANCODE_LEFTCONTROL || 
            scancode == SCANCODE_RIGHTCONTROL)
	    if (type == evKeyRelease)
		ctrl_c_state_ctrl = 0;
	    else
		ctrl_c_state_ctrl = 1;

	if (ctrl_c_state_ctrl && ctrl_c_state_c
	    && !(translatemode & DONT_CATCH_CTRLC))
	    raise(SIGINT);

	__keyboard_eventhandler(scancode ,
		    (type==evKeyRelease) ? KEY_EVENTRELEASE : KEY_EVENTPRESS);
        ztime.tv_sec=0;
        ztime.tv_usec=0; 
    } while (ggiEventPoll(MASTER_VIS,emKeyboard,&ztime) & emKeyboard);

    return eventhandled;
}

int keyboard_update(void)
{
    return keyboard_getevents(0);	/* Don't wait. */
}

void keyboard_waitforupdate(void)
{
    keyboard_getevents(1);	/* Wait for event. */
    return;
}

void keyboard_seteventhandler(void (*handler) (int, int))
{
    __keyboard_eventhandler = handler;
}



/* Default event handler. */

void keyboard_setdefaulteventhandler(void)
{
    __keyboard_eventhandler = default_handler;
}

static int checkscancode(int scancode)
{
    if (scancode < 0 || scancode >= NR_KEYS) {
	printf("svgalib: keyboard scancode out of range (%d).\n",
	       scancode);
	return 1;
    }
    return 0;
}

static void default_handler(int scancode, int newstate)
{
    if (checkscancode(scancode))
	return;
    if (translatemode & TRANSLATE_CURSORKEYS)
	/* Map cursor key block to keypad cursor keys. */
	switch (scancode) {
	case SCANCODE_CURSORBLOCKUP:
	    scancode = SCANCODE_CURSORUP;
	    break;
	case SCANCODE_CURSORBLOCKLEFT:
	    scancode = SCANCODE_CURSORLEFT;
	    break;
	case SCANCODE_CURSORBLOCKRIGHT:
	    scancode = SCANCODE_CURSORRIGHT;
	    break;
	case SCANCODE_CURSORBLOCKDOWN:
	    scancode = SCANCODE_CURSORDOWN;
	    break;
	}
    if (translatemode & TRANSLATE_DIAGONAL) {
	/* Translate diagonal keypad keys to two keypad cursor keys. */
	switch (scancode) {
	case SCANCODE_CURSORUPLEFT:
	    state[SCANCODE_CURSORUP] = newstate;
	    state[SCANCODE_CURSORLEFT] = newstate;
	    return;
	case SCANCODE_CURSORUPRIGHT:
	    state[SCANCODE_CURSORUP] = newstate;
	    state[SCANCODE_CURSORRIGHT] = newstate;
	    return;
	case SCANCODE_CURSORDOWNLEFT:
	    state[SCANCODE_CURSORDOWN] = newstate;
	    state[SCANCODE_CURSORLEFT] = newstate;
	    return;
	case SCANCODE_CURSORDOWNRIGHT:
	    state[SCANCODE_CURSORDOWN] = newstate;
	    state[SCANCODE_CURSORRIGHT] = newstate;
	    return;
	}
    }
    if ((translatemode & TRANSLATE_KEYPADENTER) && scancode ==
	SCANCODE_KEYPADENTER)
	scancode = SCANCODE_ENTER;

    state[scancode] = newstate;
}

void keyboard_clearstate(void)
{
    memset(state, 0, NR_KEYS);
    ctrl_c_state_c = 0;
    ctrl_c_state_ctrl = 0;
}

int keyboard_keypressed(int scancode)
{
    if (checkscancode(scancode))
	return 0;
    return state[scancode];
}

char *
 keyboard_getstate(void)
{
    return state;
}

void keyboard_translatekeys(int mode)
{
    translatemode = mode;
}
